@@ -30,11 +30,6 @@
 #include <type_traits>
 #include <utility>
 
-#include "absl/container/inlined_vector.h"
-#include "absl/functional/function_ref.h"
-#include "absl/meta/type_traits.h"
-#include "absl/strings/numbers.h"
-#include "absl/strings/string_view.h"
 #include "src/core/call/custom_metadata.h"
 #include "src/core/call/metadata_compression_traits.h"
 #include "src/core/call/parsed_metadata.h"
@@ -49,6 +44,11 @@
 #include "src/core/util/packed_table.h"
 #include "src/core/util/time.h"
 #include "src/core/util/type_list.h"
+#include "absl/container/inlined_vector.h"
+#include "absl/functional/function_ref.h"
+#include "absl/meta/type_traits.h"
+#include "absl/strings/numbers.h"
+#include "absl/strings/string_view.h"
 
 namespace grpc_core {
 
@@ -630,6 +630,8 @@ struct GrpcTarPit {
   static absl::string_view DisplayValue(Empty) { return "tarpit"; }
 };
 
+bool IsMetadataKeyAllowedInDebugOutput(absl::string_view key);
+
 namespace metadata_detail {
 
 // Build a key/value formatted debug string.
@@ -646,7 +648,6 @@ class DebugStringBuilder {
   std::string TakeOutput() { return std::move(out_); }
 
  private:
-  bool IsAllowListed(absl::string_view key) const;
   void Add(absl::string_view key, absl::string_view value);
   std::string out_;
 };
