@@ -19,10 +19,9 @@
 
 #include <memory>
 #include <tuple>
+#include <type_traits>
 #include <vector>
 
-#include "absl/container/flat_hash_set.h"
-#include "absl/status/status.h"
 #include "google/protobuf/any.upb.h"
 #include "google/protobuf/any.upbdefs.h"
 #include "google/protobuf/timestamp.upb.h"
@@ -31,6 +30,7 @@
 #include "src/core/lib/debug/trace.h"
 #include "src/core/util/function_signature.h"
 #include "src/core/util/json/json_writer.h"
+#include "src/core/util/latent_see.h"
 #include "src/core/util/memory_usage.h"
 #include "src/core/util/single_set_ptr.h"
 #include "src/core/util/string.h"
@@ -39,6 +39,9 @@
 #include "src/proto/grpc/channelz/v2/channelz.upb.h"
 #include "src/proto/grpc/channelz/v2/service.upb.h"
 #include "upb/mem/arena.hpp"
+#include "absl/container/flat_hash_set.h"
+#include "absl/meta/type_traits.h"
+#include "absl/status/status.h"
 
 #ifdef GRPC_NO_ZTRACE
 namespace grpc_core::channelz {
@@ -61,6 +64,8 @@ class StubImpl {
   std::unique_ptr<ZTrace> MakeZTrace() {
     return std::make_unique<ZTraceImpl>();
   }
+
+  bool IsActive() { return false; }
 };
 }  // namespace ztrace_collector_detail
 
@@ -163,6 +168,13 @@ class ZTraceCollector {
   // complexity.
   template <typename X>
   void Append(X producer_or_value) {
+    if constexpr (ztrace_collector_detail::kIsElement<X, Data...>) {
+      GRPC_LATENT_SEE_ALWAYS_ON_MARK_EXTRA_EVENT(X, producer_or_value);
+    } else {
+      using ResultType = absl::result_of_t<decltype(producer_or_value)()>;
+      GRPC_LATENT_SEE_ALWAYS_ON_MARK_EXTRA_EVENT(ResultType,
+                                                 producer_or_value());
+    }
     GRPC_TRACE_LOG(ztrace, INFO) << "ZTRACE[" << this << "]: " << [&]() {
       upb::Arena arena;
       google_protobuf_Any* any = google_protobuf_Any_new(arena.ptr());
