@@ -22,16 +22,20 @@
 #include <variant>
 #include <vector>
 
-#include "absl/status/statusor.h"
-#include "absl/strings/string_view.h"
-#include "absl/types/span.h"
 #include "src/core/ext/transport/chttp2/transport/http2_settings.h"
 #include "src/core/ext/transport/chttp2/transport/http2_status.h"
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/slice/slice_buffer.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 
 namespace grpc_core {
 
+// Prints the first `length` bytes of the payload. If the payload is longer than
+// `length`, it appends "<clipped>" to the output.
+std::string MaybeTruncatePayload(SliceBuffer& payload, uint32_t length = 15);
+
 ///////////////////////////////////////////////////////////////////////////////
 // Frame types
 //
@@ -224,6 +228,10 @@ struct Http2FrameHeader {
 ///////////////////////////////////////////////////////////////////////////////
 // Parsing & serialization
 
+struct SerializeReturn {
+  bool should_reset_ping_clock;
+};
+
 // Given a frame header and a payload, parse the payload into a frame and
 // return it.
 // If this function returns an error, that should be considered a connection
@@ -236,27 +244,34 @@ http2::ValueOrHttp2Status<Http2Frame> ParseFramePayload(
 
 // Serialize frame and append to out, leaves frames in an unknown state (may
 // move things out of frames)
-void Serialize(absl::Span<Http2Frame> frames, SliceBuffer& out);
+SerializeReturn Serialize(absl::Span<Http2Frame> frames, SliceBuffer& out);
 
-http2::Http2ErrorCode Http2ErrorCodeFromRstFrameErrorCode(uint32_t error_code);
+http2::Http2ErrorCode FrameErrorCodeToHttp2ErrorCode(uint32_t error_code);
+uint32_t Http2ErrorCodeToFrameErrorCode(http2::Http2ErrorCode error_code);
+
+// Returns approximate memory usage of the frame.
+size_t GetFrameMemoryUsage(const Http2Frame& frame);
 
 ///////////////////////////////////////////////////////////////////////////////
 // GRPC Header
 
 constexpr uint8_t kGrpcHeaderSizeInBytes = 5;
+constexpr uint8_t kGrpcMessageHeaderNoFlags = 0;
+constexpr uint8_t kGrpcMessageHeaderWriteInternalCompress = 1;
 
 struct GrpcMessageHeader {
-  uint8_t flags = 0;
+  uint32_t flags = 0;
   uint32_t length = 0;
 };
 
 // If the payload SliceBuffer is too small to hold a gRPC header, this function
 // will crash. The calling function MUST ensure that the payload SliceBuffer
 // has length greater than or equal to the gRPC header.
-GrpcMessageHeader ExtractGrpcHeader(SliceBuffer& payload);
+http2::ValueOrHttp2Status<GrpcMessageHeader> ExtractGrpcHeader(
+    SliceBuffer& payload);
 
-void AppendGrpcHeaderToSliceBuffer(SliceBuffer& payload, const uint8_t flags,
-                                   const uint32_t length);
+void AppendGrpcHeaderToSliceBuffer(SliceBuffer& payload, uint32_t flags,
+                                   uint32_t length);
 
 ///////////////////////////////////////////////////////////////////////////////
 // Validations
@@ -267,7 +282,9 @@ http2::Http2Status ValidateSettingsValues(
 http2::Http2Status ValidateFrameHeader(uint32_t max_frame_size_setting,
                                        bool incoming_header_in_progress,
                                        uint32_t incoming_header_stream_id,
-                                       Http2FrameHeader& current_frame_header);
+                                       Http2FrameHeader& current_frame_header,
+                                       uint32_t last_stream_id, bool is_client,
+                                       bool is_first_settings_processed);
 
 ///////////////////////////////////////////////////////////////////////////////
 // RFC9113 Related Strings and Consts
@@ -301,6 +318,9 @@ inline constexpr absl::string_view kSettingsStreamIdMustBeZero =
     "RFC9113: If an endpoint receives a SETTINGS frame whose Stream "
     "Identifier field is anything other than 0x00, the endpoint MUST respond "
     "with a connection error";
+inline constexpr absl::string_view kUnknownStreamId =
+    "RFC9113: An endpoint that receives an unexpected stream identifier MUST "
+    "respond with a connection error";
 
 // Frame length related errors
 inline constexpr absl::string_view kRstStreamLength4 =
@@ -357,13 +377,41 @@ inline constexpr absl::string_view kIncorrectFrameSizeSetting =
     "RFC9113: The initial value is 2^14 (16,384) octets. The value advertised"
     " by an endpoint MUST be between this initial value and the maximum allowed"
     " frame size (2^24)-1 or 16,777,215 octets), inclusive. ";
+inline constexpr absl::string_view kSettingsTimeout =
+    "Settings timeout. The HTTP2 settings frame was not ACKed within the "
+    "timeout. Connection will be closed";
+inline constexpr absl::string_view kLastStreamClosed =
+    "Closing last stream and cannot create any more streams.";
+inline constexpr absl::string_view kFirstSettingsFrameClient =
+    "RFC9113: The server connection preface consists of a potentially empty "
+    "SETTINGS frame that MUST be the first frame the server sends in the "
+    "HTTP/2 connection. Clients and servers MUST treat an invalid connection "
+    "preface as a connection error of type PROTOCOL_ERROR.";
+inline constexpr absl::string_view kFirstSettingsFrameServer =
+    "RFC9113: The client connection preface starts with a sequence of 24 "
+    "octets. This sequence is followed by a SETTINGS frame, which MAY be "
+    "empty. Clients and servers MUST treat an invalid connection preface as a "
+    "connection error of type PROTOCOL_ERROR.";
 
 inline constexpr uint32_t kMaxStreamId31Bit = 0x7fffffffu;
 inline constexpr uint32_t kMaxSize31Bit = 0x7fffffffu;
 inline constexpr uint32_t kMinimumFrameSize = 16384;
 inline constexpr uint32_t kMaximumFrameSize = 16777215;
 
+// Default Initial Window Size as per RFC 9113 Section 6.5.2
+inline constexpr uint32_t kHttp2InitialWindowSize = 65535u;
+
 }  // namespace RFC9113
+
+namespace GrpcErrors {
+inline constexpr absl::string_view kTooManyMetadata =
+    "gRPC Error : A gRPC server can send upto 1 initial metadata followed by "
+    "upto 1 trailing metadata.";
+inline constexpr absl::string_view kOutOfOrderDataFrame =
+    "gRPC Error : DATA frames must follow initial metadata and precede "
+    "trailing metadata.";
+}  // namespace GrpcErrors
+
 }  // namespace grpc_core
 
 #endif  // GRPC_SRC_CORE_EXT_TRANSPORT_CHTTP2_TRANSPORT_FRAME_H
