@@ -17,12 +17,17 @@
 //
 #include "src/core/ext/transport/chttp2/transport/ping_promise.h"
 
+#include <memory>
+#include <utility>
+
+#include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/promise/latch.h"
 #include "src/core/lib/promise/map.h"
 #include "src/core/lib/promise/party.h"
 #include "src/core/lib/promise/race.h"
 #include "src/core/lib/promise/sleep.h"
 #include "src/core/lib/promise/try_seq.h"
+#include "src/core/util/grpc_check.h"
 #include "src/core/util/match.h"
 #include "src/core/util/time.h"
 
@@ -54,13 +59,14 @@ Promise<absl::Status> PingManager::PingPromiseCallbacks::WaitForPingAck() {
 }
 
 // Ping System implementation
-PingManager::PingManager(const ChannelArgs& channel_args,
+PingManager::PingManager(const ChannelArgs& channel_args, Duration ping_timeout,
                          std::unique_ptr<PingInterface> ping_interface,
                          std::shared_ptr<EventEngine> event_engine)
     : ping_callbacks_(event_engine),
       ping_abuse_policy_(channel_args),
       ping_rate_policy_(channel_args, /*is_client=*/true),
-      ping_interface_(std::move(ping_interface)) {}
+      ping_interface_(std::move(ping_interface)),
+      ping_timeout_(ping_timeout) {}
 
 void PingManager::TriggerDelayedPing(const Duration wait) {
   // Spawn at most once.
@@ -146,7 +152,7 @@ void PingManager::MaybeGetSerializedPingFrames(
                       << pending_ping_acks_.size()
                       << " next_allowed_ping_interval: "
                       << next_allowed_ping_interval;
-  DCHECK(!opaque_data_.has_value());
+  GRPC_DCHECK(!opaque_data_.has_value());
   std::vector<Http2Frame> frames;
   frames.reserve(pending_ping_acks_.size() + 1);  // +1 for the ping frame.
 
@@ -170,9 +176,9 @@ void PingManager::MaybeGetSerializedPingFrames(
   }
 }
 
-void PingManager::NotifyPingSent(const Duration ping_timeout) {
+void PingManager::NotifyPingSent() {
   if (opaque_data_.has_value()) {
-    SpawnTimeout(ping_timeout, opaque_data_.value());
+    SpawnTimeout(ping_timeout_, opaque_data_.value());
     SentPing();
     opaque_data_.reset();
   }
