@@ -23,13 +23,6 @@
 #include <utility>
 #include <vector>
 
-#include "absl/base/attributes.h"
-#include "absl/functional/function_ref.h"
-#include "absl/log/log.h"
-#include "absl/status/status.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/str_format.h"
-#include "absl/strings/str_join.h"
 #include "src/core/lib/channel/channel_stack.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/iomgr/error.h"
@@ -40,6 +33,13 @@
 #include "src/core/util/latent_see.h"
 #include "src/core/util/manual_constructor.h"
 #include "src/core/util/status_helper.h"
+#include "absl/base/attributes.h"
+#include "absl/functional/function_ref.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_join.h"
 
 namespace grpc_core {
 namespace promise_filter_detail {
@@ -84,7 +84,13 @@ BaseCallData::BaseCallData(
     grpc_call_element* elem, const grpc_call_element_args* args, uint8_t flags,
     absl::FunctionRef<Interceptor*()> make_send_interceptor,
     absl::FunctionRef<Interceptor*()> make_recv_interceptor)
-    : call_stack_(args->call_stack),
+    : channelz::DataSource([args]() -> RefCountedPtr<channelz::BaseNode> {
+        channelz::CallNode* call_node =
+            args->arena->GetContext<channelz::CallNode>();
+        if (call_node == nullptr) return nullptr;
+        return call_node->Ref();
+      }()),
+      call_stack_(args->call_stack),
       elem_(elem),
       arena_(args->arena),
       call_combiner_(args->call_combiner),
@@ -149,6 +155,37 @@ std::string BaseCallData::LogTag() const {
       absl::Hex(reinterpret_cast<uintptr_t>(elem_), absl::kZeroPad8), "]");
 }
 
+void BaseCallData::AddData(channelz::DataSink sink) {
+  EnsureRunInExecCtx([this, sink = std::move(sink)]() mutable {
+    auto add = [sink, this](grpc_error_handle) mutable {
+      sink.AddData(elem_->filter->name.name(), ChannelzProperties());
+      GRPC_CALL_COMBINER_STOP(call_combiner(), "channelz_add_data");
+      GRPC_CALL_STACK_UNREF(call_stack_, "channelz_add_data");
+    };
+    GRPC_CALL_STACK_REF(call_stack_, "channelz_add_data");
+    GRPC_CALL_COMBINER_START(call_combiner_, NewClosure(std::move(add)),
+                             absl::OkStatus(), "channelz_add_data");
+  });
+}
+
+channelz::PropertyList BaseCallData::ChannelzProperties() const {
+  channelz::PropertyList properties;
+  properties.Set("deadline", deadline_);
+  if (send_message_ != nullptr) {
+    properties.Set("promise_based_send_message",
+                   send_message_->ChannelzProperties());
+  }
+  if (receive_message_ != nullptr) {
+    properties.Set("promise_based_receive_message",
+                   receive_message_->ChannelzProperties());
+  }
+  if (server_initial_metadata_pipe_ != nullptr) {
+    properties.Set("server_initial_metadata_pipe",
+                   server_initial_metadata_pipe_->sender.ChannelzProperties());
+  }
+  return properties;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // BaseCallData::CapturedBatch
 
@@ -1270,9 +1307,11 @@ ClientCallData::ClientCallData(grpc_call_element* elem,
   if (server_initial_metadata_pipe() != nullptr) {
     recv_initial_metadata_ = arena()->New<RecvInitialMetadata>();
   }
+  SourceConstructed();
 }
 
 ClientCallData::~ClientCallData() {
+  SourceDestructing();
   ScopedActivity scoped_activity(this);
   GRPC_CHECK_EQ(poll_ctx_, nullptr);
   if (recv_initial_metadata_ != nullptr) {
@@ -1803,6 +1842,13 @@ void ClientCallData::OnWakeup() {
   WakeInsideCombiner(&flusher);
 }
 
+channelz::PropertyList ClientCallData::ChannelzProperties() const {
+  return BaseCallData::ChannelzProperties()
+      .Set("promise", PromiseProperty(&promise_))
+      .Set("send_initial_state", StateString(send_initial_state_))
+      .Set("recv_trailing_state", StateString(recv_trailing_state_));
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // ServerCallData
 
@@ -1952,9 +1998,11 @@ ServerCallData::ServerCallData(grpc_call_element* elem,
   GRPC_CLOSURE_INIT(&recv_trailing_metadata_ready_,
                     RecvTrailingMetadataReadyCallback, this,
                     grpc_schedule_on_exec_ctx);
+  SourceConstructed();
 }
 
 ServerCallData::~ServerCallData() {
+  SourceDestructing();
   GRPC_TRACE_LOG(channel, INFO)
       << LogTag() << " ~ServerCallData " << DebugString();
   if (send_initial_metadata_ != nullptr) {
@@ -2503,5 +2551,12 @@ void ServerCallData::OnWakeup() {
   WakeInsideCombiner(&flusher);
 }
 
+channelz::PropertyList ServerCallData::ChannelzProperties() const {
+  return BaseCallData::ChannelzProperties()
+      .Set("promise", PromiseProperty(&promise_))
+      .Set("recv_initial_state", StateString(recv_initial_state_))
+      .Set("send_trailing_state", StateString(send_trailing_state_));
+}
+
 }  // namespace promise_filter_detail
 }  // namespace grpc_core
