@@ -33,12 +33,6 @@
 #include <type_traits>
 #include <utility>
 
-#include "absl/container/inlined_vector.h"
-#include "absl/functional/function_ref.h"
-#include "absl/log/log.h"
-#include "absl/meta/type_traits.h"
-#include "absl/status/status.h"
-#include "absl/strings/string_view.h"
 #include "src/core/call/call_filters.h"
 #include "src/core/call/call_finalization.h"
 #include "src/core/call/message.h"
@@ -75,6 +69,12 @@
 #include "src/core/util/grpc_check.h"
 #include "src/core/util/match.h"
 #include "src/core/util/time.h"
+#include "absl/container/inlined_vector.h"
+#include "absl/functional/function_ref.h"
+#include "absl/log/log.h"
+#include "absl/meta/type_traits.h"
+#include "absl/status/status.h"
+#include "absl/strings/string_view.h"
 
 namespace grpc_core {
 
@@ -1248,7 +1248,9 @@ class InvalidChannelFilter : public ChannelFilter {
 };
 
 // Call data shared between all implementations of promise-based filters.
-class BaseCallData : public Activity, private Wakeable {
+class BaseCallData : public Activity,
+                     private Wakeable,
+                     public channelz::DataSource {
  protected:
   // Hook to allow interception of messages on the send/receive path by
   // PipeSender and PipeReceiver, as appropriate according to whether we're
@@ -1293,6 +1295,8 @@ class BaseCallData : public Activity, private Wakeable {
 
   Call* call() { return arena_->GetContext<Call>(); }
 
+  void AddData(channelz::DataSink sink) final;
+
  protected:
   class ScopedContext : public promise_detail::Context<Arena>,
                         public promise_detail::Context<grpc_polling_entity>,
@@ -1471,6 +1475,10 @@ class BaseCallData : public Activity, private Wakeable {
     // Return true if we've released the message for forwarding down the stack.
     bool IsForwarded() const { return state_ == State::kForwardedBatch; }
 
+    channelz::PropertyList ChannelzProperties() {
+      return channelz::PropertyList().Set("state", StateString(state_));
+    }
+
    private:
     enum class State : uint8_t {
       // Starting state: no batch started, no outgoing pipe configured.
@@ -1544,6 +1552,10 @@ class BaseCallData : public Activity, private Wakeable {
     // Call is completed, we have trailing metadata. Close things out.
     void Done(const ServerMetadata& metadata, Flusher* flusher);
 
+    channelz::PropertyList ChannelzProperties() {
+      return channelz::PropertyList().Set("state", StateString(state_));
+    }
+
    private:
     enum class State : uint8_t {
       // Starting state: no batch started, no incoming pipe configured.
@@ -1626,6 +1638,7 @@ class BaseCallData : public Activity, private Wakeable {
   }
   SendMessage* send_message() const { return send_message_; }
   ReceiveMessage* receive_message() const { return receive_message_; }
+  virtual channelz::PropertyList ChannelzProperties() const;
 
   bool is_last() const {
     return grpc_call_stack_element(call_stack_, call_stack_->count - 1) ==
@@ -1707,6 +1720,7 @@ class ClientCallData : public BaseCallData {
   static const char* StateString(SendInitialState);
   static const char* StateString(RecvTrailingState);
   std::string DebugString() const;
+  channelz::PropertyList ChannelzProperties() const override;
 
   struct RecvInitialMetadata;
   class PollContext;
@@ -1823,6 +1837,7 @@ class ServerCallData : public BaseCallData {
   static const char* StateString(RecvInitialState state);
   static const char* StateString(SendTrailingState state);
   std::string DebugString() const;
+  channelz::PropertyList ChannelzProperties() const override;
 
   class PollContext;
   struct SendInitialMetadata;
@@ -1985,7 +2000,7 @@ struct ChannelFilterWithFlagsMethods {
         F::Create(args->channel_args,
                   ChannelFilter::Args(args->channel_stack, elem,
                                       grpc_channel_stack_filter_instance_number,
-                                      args->blackboard));
+                                      args->config, args->blackboard));
     if (!status.ok()) {
       new (elem->channel_data) F*(nullptr);
       return absl_status_to_grpc_error(status.status());
