@@ -34,10 +34,6 @@
 
 #include <string>
 
-#include "absl/log/check.h"
-#include "absl/log/log.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/str_format.h"
 #include "src/core/lib/event_engine/shim.h"
 #include "src/core/lib/experiments/experiments.h"
 #include "src/core/lib/iomgr/block_annotate.h"
@@ -47,8 +43,12 @@
 #include "src/core/telemetry/stats.h"
 #include "src/core/telemetry/stats_data.h"
 #include "src/core/util/crash.h"
+#include "src/core/util/grpc_check.h"
 #include "src/core/util/thd.h"
 #include "src/core/util/useful.h"
+#include "absl/log/log.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
 
 #define GRPC_POLLSET_KICK_BROADCAST ((grpc_pollset_worker*)1)
 
@@ -350,7 +350,7 @@ static void ref_by(grpc_fd* fd, int n, const char* reason, const char* file,
   } while (0)
 static void ref_by(grpc_fd* fd, int n) {
 #endif
-  CHECK_GT(gpr_atm_no_barrier_fetch_add(&fd->refst, n), 0);
+  GRPC_CHECK_GT(gpr_atm_no_barrier_fetch_add(&fd->refst, n), 0);
 }
 
 #ifndef NDEBUG
@@ -374,7 +374,7 @@ static void unref_by(grpc_fd* fd, int n) {
     fd->shutdown_error.~Status();
     gpr_free(fd);
   } else {
-    CHECK(old > n);
+    GRPC_CHECK(old > n);
   }
 }
 
@@ -392,7 +392,7 @@ static grpc_fd* fd_create(int fd, const char* name, bool track_err) {
   }
   // Avoid unused-parameter warning for debug-only parameter
   (void)track_err;
-  DCHECK(track_err == false);
+  GRPC_DCHECK(track_err == false);
   grpc_fd* r = static_cast<grpc_fd*>(gpr_malloc(sizeof(*r)));
   gpr_mu_init(&r->mu);
   gpr_atm_rel_store(&r->refst, 1);
@@ -422,7 +422,7 @@ static bool fd_is_orphaned(grpc_fd* fd) {
 
 static grpc_error_handle pollset_kick_locked(grpc_fd_watcher* watcher) {
   gpr_mu_lock(&watcher->pollset->mu);
-  CHECK(watcher->worker);
+  GRPC_CHECK(watcher->worker);
   grpc_error_handle err =
       pollset_kick_ext(watcher->pollset, watcher->worker,
                        GRPC_POLLSET_REEVALUATE_POLLING_ON_WAKEUP);
@@ -483,8 +483,8 @@ static void fd_orphan(grpc_fd* fd, grpc_closure* on_done, int* release_fd,
   if (grpc_core::IsEventEngineForAllOtherEndpointsEnabled() &&
       !grpc_event_engine::experimental::
           EventEngineExperimentDisabledForPython()) {
-    CHECK_NE(release_fd, nullptr);
-    CHECK_EQ(on_done, nullptr);
+    GRPC_CHECK_NE(release_fd, nullptr);
+    GRPC_CHECK_EQ(on_done, nullptr);
     *release_fd = fd->fd;
     gpr_free(fd);
     return;
@@ -805,7 +805,7 @@ static grpc_error_handle pollset_kick_ext(grpc_pollset* p,
   // pollset->mu already held
   if (specific_worker != nullptr) {
     if (specific_worker == GRPC_POLLSET_KICK_BROADCAST) {
-      CHECK_EQ((flags & GRPC_POLLSET_REEVALUATE_POLLING_ON_WAKEUP), 0u);
+      GRPC_CHECK_EQ((flags & GRPC_POLLSET_REEVALUATE_POLLING_ON_WAKEUP), 0u);
       for (specific_worker = p->root_worker.next;
            specific_worker != &p->root_worker;
            specific_worker = specific_worker->next) {
@@ -829,7 +829,7 @@ static grpc_error_handle pollset_kick_ext(grpc_pollset* p,
                         grpc_wakeup_fd_wakeup(&specific_worker->wakeup_fd->fd));
     }
   } else if (g_current_thread_poller != p) {
-    CHECK_EQ((flags & GRPC_POLLSET_REEVALUATE_POLLING_ON_WAKEUP), 0u);
+    GRPC_CHECK_EQ((flags & GRPC_POLLSET_REEVALUATE_POLLING_ON_WAKEUP), 0u);
     specific_worker = pop_front_worker(p);
     if (specific_worker != nullptr) {
       if (g_current_thread_worker == specific_worker) {
@@ -882,7 +882,7 @@ static void pollset_init(grpc_pollset* pollset, gpr_mu** mu) {
 }
 
 static void pollset_destroy(grpc_pollset* pollset) {
-  CHECK(!pollset_has_workers(pollset));
+  GRPC_CHECK(!pollset_has_workers(pollset));
   while (pollset->local_wakeup_cache) {
     grpc_cached_wakeup_fd* next = pollset->local_wakeup_cache->next;
     fork_fd_list_remove_wakeup_fd(pollset->local_wakeup_cache);
@@ -1160,7 +1160,7 @@ static grpc_error_handle pollset_work(grpc_pollset* pollset,
 }
 
 static void pollset_shutdown(grpc_pollset* pollset, grpc_closure* closure) {
-  CHECK(!pollset->shutting_down);
+  GRPC_CHECK(!pollset->shutting_down);
   pollset->shutting_down = 1;
   pollset->shutdown_done = closure;
   (void)pollset_kick(pollset, GRPC_POLLSET_KICK_BROADCAST);
