@@ -33,16 +33,13 @@
 #include <string>
 #include <utility>
 
-#include "absl/functional/any_invocable.h"
-#include "absl/log/log.h"
-#include "absl/status/status.h"
-#include "absl/strings/string_view.h"
 #include "src/core/call/call_destination.h"
 #include "src/core/call/call_spine.h"
 #include "src/core/call/message.h"
 #include "src/core/call/metadata.h"
 #include "src/core/call/metadata_batch.h"
 #include "src/core/channelz/channelz.h"
+#include "src/core/ext/transport/chttp2/transport/http2_status.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/iomgr/call_combiner.h"
 #include "src/core/lib/iomgr/closure.h"
@@ -61,6 +58,10 @@
 #include "src/core/lib/transport/transport_fwd.h"
 #include "src/core/util/orphanable.h"
 #include "src/core/util/ref_counted.h"
+#include "absl/functional/any_invocable.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/strings/string_view.h"
 
 // Minimum and maximum protocol accepted versions.
 #define GRPC_PROTOCOL_VERSION_MAX_MAJOR 2
@@ -507,6 +508,58 @@ class ServerTransport;
 
 class Transport : public InternallyRefCounted<Transport> {
  public:
+  // An interface used by channels or servers to watch the transport's state.
+  class StateWatcher : public RefCounted<StateWatcher> {
+   public:
+    ~StateWatcher() override = default;
+
+    // The list of reasons is defined in
+    // https://github.com/grpc/proposal/blob/master/A94-subchannel-otel-metrics.md.
+    // Note that we do not include the "subchannel shutdown" reason
+    // here, since that reason is not generated by the transport.
+    enum DisconnectReason {
+      kUnknown,
+      kGoaway,
+      kConnectionReset,
+      kConnectionTimedOut,
+      kConnectionAborted,
+      kSocketError,
+    };
+
+    struct DisconnectInfo {
+      DisconnectReason reason = kUnknown;
+      std::optional<http2::Http2ErrorCode> http2_error_code;
+      std::optional<Duration> keepalive_time;
+    };
+
+    // Called on disconnection or GOAWAY.  The channel or server must
+    // stop sending traffic to this transport.  The transport will
+    // automatically stop the watch after this.
+    virtual void OnDisconnect(absl::Status status,
+                              DisconnectInfo disconnect_info) = 0;
+
+    // A handle passed to the subchannel by the transport via
+    // OnPeerMaxConcurrentStreamsUpdate().  The subchannel must delete
+    // this handle when it has finished processing the update.
+    class MaxConcurrentStreamsUpdateDoneHandle {
+     public:
+      virtual ~MaxConcurrentStreamsUpdateDoneHandle() = default;
+    };
+
+    // Used on client transports only.
+    // Will be called once as soon as the watch is started to indicate
+    // the current value of the peer's MAX_CONCURRENT_STREAMS setting.
+    // Will then be called again whenever the peer changes this setting.
+    // The on_done handle must be deleted when the implementation is
+    // done processing the update.
+    virtual void OnPeerMaxConcurrentStreamsUpdate(
+        uint32_t max_concurrent_streams,
+        std::unique_ptr<MaxConcurrentStreamsUpdateDoneHandle> on_done) = 0;
+
+    // TODO(roth): Remove this as part of the EventEngine migration.
+    virtual grpc_pollset_set* interested_parties() const = 0;
+  };
+
   struct RawPointerChannelArgTag {};
   static absl::string_view ChannelArgName() { return GRPC_ARG_TRANSPORT; }
 
@@ -547,6 +600,8 @@ class Transport : public InternallyRefCounted<Transport> {
   // implementation of grpc_transport_perform_op
   virtual void PerformOp(grpc_transport_op* op) = 0;
 
+  // TODO(roth, ctiller): Migrate all callers to the new StartWatch()
+  // API and remove this.
   void StartConnectivityWatch(
       OrphanablePtr<ConnectivityStateWatcherInterface> watcher) {
     grpc_transport_op* op = grpc_make_transport_op(nullptr);
@@ -561,6 +616,9 @@ class Transport : public InternallyRefCounted<Transport> {
     PerformOp(op);
   }
 
+  virtual void StartWatch(RefCountedPtr<StateWatcher> watcher) = 0;
+  virtual void StopWatch(RefCountedPtr<StateWatcher> watcher) = 0;
+
   virtual RefCountedPtr<channelz::SocketNode> GetSocketNode() const = 0;
 };
 
