@@ -80,16 +80,6 @@
 #include <variant>
 #include <vector>
 
-#include "absl/container/inlined_vector.h"
-#include "absl/functional/function_ref.h"
-#include "absl/log/globals.h"
-#include "absl/log/log.h"
-#include "absl/status/status.h"
-#include "absl/status/statusor.h"
-#include "absl/strings/str_cat.h"
-#include "absl/strings/str_format.h"
-#include "absl/strings/str_join.h"
-#include "absl/strings/string_view.h"
 #include "src/core/call/metadata_batch.h"
 #include "src/core/channelz/channelz.h"
 #include "src/core/client_channel/client_channel_filter.h"
@@ -141,6 +131,16 @@
 #include "src/core/util/validation_errors.h"
 #include "src/core/util/work_serializer.h"
 #include "upb/mem/arena.hpp"
+#include "absl/container/inlined_vector.h"
+#include "absl/functional/function_ref.h"
+#include "absl/log/globals.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
+#include "absl/strings/str_join.h"
+#include "absl/strings/string_view.h"
 
 #define GRPC_GRPCLB_INITIAL_CONNECT_BACKOFF_SECONDS 1
 #define GRPC_GRPCLB_RECONNECT_BACKOFF_MULTIPLIER 1.6
@@ -417,10 +417,9 @@ class GrpcLb final : public LoadBalancingPolicy {
     PickResult Pick(PickArgs args) override;
 
    private:
-    // A subchannel call tracker that unrefs the GrpcLbClientStats object
-    // in the case where the subchannel call is never actually started,
-    // since the client load reporting filter will not be able to do it
-    // in that case.
+    // A subchannel call tracker that holds a ref to the
+    // GrpcLbClientStats object, to ensure that it still exists when the
+    // client load reporting filter sees it and takes its own ref to it.
     class SubchannelCallTracker final : public SubchannelCallTrackerInterface {
      public:
       SubchannelCallTracker(
@@ -429,16 +428,6 @@ class GrpcLb final : public LoadBalancingPolicy {
           : client_stats_(std::move(client_stats)),
             original_call_tracker_(std::move(original_call_tracker)) {}
 
-      void Start() override {
-        if (original_call_tracker_ != nullptr) {
-          original_call_tracker_->Start();
-        }
-        // If we're actually starting the subchannel call, then the
-        // client load reporting filter will take ownership of the ref
-        // passed down to it via metadata.
-        client_stats_.release();
-      }
-
       void Finish(FinishArgs args) override {
         if (original_call_tracker_ != nullptr) {
           original_call_tracker_->Finish(args);
